<div class='resizer layer'></div>
<div class='console-outer layer'>
  <div class='console-actions'>
    <div class='close-button button' title='close'>x</div>
  </div>
  <div class='console-inner'></div>
</div>
<input class='clipboard' type='text'>
